package ram.kulkarni.war.sql.demo;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

/**
 * 
 * @author Ram Kulkarni (ramkulkarni.com)
 *
 */
public class EmployeeDBManager extends SQLiteManger {

	public EmployeeDBManager(String dbPath) throws Exception
	{
		super(dbPath);
		createTables();
	}
	
	private void createTables() throws SQLException
	{
		Statement stmt = con.createStatement();
		
		String sql = "create table if not exists employee (id integer primary key autoincrement not null" +
				", first_name text, last_name text, address text)";
		
		stmt.execute(sql);
	}

	public int addEmployee (EmployeeVO vo) throws SQLException
	{
		return addEmployee(vo.firstName, vo.lastName, vo.address);
	}
	
	public int addEmployee (String firstName, String lastName, String address) throws SQLException
	{
		String sql = "insert into employee (first_name, last_name, address) values (?,?,?)";
		
		PreparedStatement stmt = con.prepareStatement(sql);
		stmt.setString(1, firstName);
		stmt.setString(2, lastName);
		stmt.setString(3, address);
		
		stmt.execute();
		stmt.close();
		
		//get id of new employee
		Statement stmt1 = con.createStatement();
		ResultSet rs = stmt1.executeQuery("SELECT last_insert_rowid()");
		
		int newId = -1;
		
		if (rs.next())
			newId = rs.getInt(1);
		
		stmt1.close();

		return newId;
	}

	public void deleteEmployee (int id) throws SQLException
	{
		Statement stmt = con.createStatement();
		stmt.execute("delete from employee where id = " + id);
		stmt.close();
	}
	
	public void updateEmployee (EmployeeVO vo) throws SQLException
	{
		updateEmployee(vo.id, vo.firstName, vo.lastName, vo.address);
	}
	
	public void updateEmployee (int id, String firstName, String lastName, String address) throws SQLException
	{
		String sql = "update employee set first_name = ?, last_name = ?, address = ? " + 
					"where id = " + id;
		
		PreparedStatement stmt = con.prepareStatement(sql);
		stmt.setString(1, firstName);
		stmt.setString(2, lastName);
		stmt.setString(3, address);
		
		stmt.execute();
		
		stmt.close();
	}
	
	public EmployeeVO[] getEmployees() throws SQLException
	{
		String sql = "select * from employee order by first_name,last_name";
		
		Statement stmt = con.createStatement();
		
		ResultSet rs = stmt.executeQuery(sql);
		
		ArrayList<EmployeeVO> result = new ArrayList<EmployeeVO>();
		
		while (rs.next())
			result.add(rowToEmployeeVO(rs));
		
		stmt.close();
		
		return result.toArray(new EmployeeVO[]{});
	}
	
	public EmployeeVO getEmployee(int id) throws SQLException
	{
		String sql = "select * from employee where id = " + id;
		
		Statement stmt = con.createStatement();
		
		ResultSet rs = stmt.executeQuery(sql);
		
		EmployeeVO result = null;
		
		if (rs.next())
			result = rowToEmployeeVO(rs);
		
		stmt.close();
		
		return result;
	}
	
	private EmployeeVO rowToEmployeeVO (ResultSet rs) throws SQLException
	{
		EmployeeVO emp = new EmployeeVO();
		emp.id = rs.getInt("id");
		emp.firstName = rs.getString("first_name");
		emp.lastName = rs.getString("last_name");
		emp.address = rs.getString("address");
		return emp;
	}
}
